@@ -3,7 +3,7 @@
     SPDX-FileCopyrightText: 2005-2006 Olivier Goffart <ogoffart at kde.org>
     SPDX-FileCopyrightText: 2013-2015 Martin Klapetek <mklapetek@kde.org>
 
-    SPDX-License-Identifier: LGPL-2.0-only
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KNOTIFICATION_H
@@ -11,14 +11,81 @@
 
 #include <knotifications_export.h>
 
-#include <QPixmap>
-#include <QObject>
 #include <QList>
+#include <QObject>
 #include <QPair>
+#include <QPixmap>
 #include <QUrl>
 #include <QVariant>
+#include <QWindow>
 
-class QWidget;
+#include <memory>
+
+class KNotificationReplyAction;
+class KNotificationAction;
+
+class KNotificationActionPrivate;
+
+/**
+ * @class KNotificationAction knotification.h KNotificationAction
+ *
+ * This class represents a notification. This can be a button on the notification
+ * popup, or triggered by clicking the notification popup itself.
+ *
+ * @since 6.0
+ */
+class KNOTIFICATIONS_EXPORT KNotificationAction : public QObject
+{
+    Q_OBJECT
+    /**
+     * @copydoc label
+     */
+    Q_PROPERTY(QString label READ label WRITE setLabel NOTIFY labelChanged)
+
+public:
+    explicit KNotificationAction(QObject *parent = nullptr);
+
+    /**
+     * Creates an action with given label
+     * @param label The label for the action
+     */
+    explicit KNotificationAction(const QString &label);
+
+    ~KNotificationAction() override;
+
+    /**
+     * The user-facing label for the action
+     */
+    QString label() const;
+
+    /**
+     * Set the user-facing label for the action
+     */
+    void setLabel(const QString &label);
+
+Q_SIGNALS:
+    /**
+     * Emitted when the user activates the action
+     */
+    void activated();
+
+    /**
+     * Emitted when @p label changed.
+     */
+    void labelChanged(const QString &label);
+
+private:
+    friend class KNotification;
+    friend class NotifyByPortalPrivate;
+    friend class NotifyByPopup;
+    friend class NotifyBySnore;
+    friend class NotifyByAndroid;
+
+    void setId(const QString &id);
+    QString id() const;
+
+    std::unique_ptr<KNotificationActionPrivate> const d;
+};
 
 /**
  * @class KNotification knotification.h KNotification
@@ -28,41 +95,66 @@ class QWidget;
 class KNOTIFICATIONS_EXPORT KNotification : public QObject
 {
     Q_OBJECT
+    /**
+     * @copydoc setEventId
+     * @since 5.88
+     */
+    Q_PROPERTY(QString eventId READ eventId WRITE setEventId NOTIFY eventIdChanged)
+    /**
+     * @copydoc setTitle
+     * @since 5.88
+     */
+    Q_PROPERTY(QString title READ title WRITE setTitle NOTIFY titleChanged)
+    /**
+     * @copydoc setText
+     * @since 5.88
+     */
+    Q_PROPERTY(QString text READ text WRITE setText NOTIFY textChanged)
+    /**
+     * @copydoc setIconName
+     * @since 5.88
+     */
+    Q_PROPERTY(QString iconName READ iconName WRITE setIconName NOTIFY iconNameChanged)
+    /**
+     * @copydoc setFlags
+     * @since 5.88
+     */
+    Q_PROPERTY(NotificationFlags flags READ flags WRITE setFlags NOTIFY flagsChanged)
+    /**
+     * @copydoc setComponentName
+     * @since 5.88
+     */
+    Q_PROPERTY(QString componentName READ componentName WRITE setComponentName NOTIFY componentNameChanged)
+    /**
+     * @copydoc setUrls
+     * @since 5.88
+     */
+    Q_PROPERTY(QList<QUrl> urls READ urls WRITE setUrls NOTIFY urlsChanged)
+    /**
+     * @copydoc setUrgency
+     * @since 5.88
+     */
+    Q_PROPERTY(Urgency urgency READ urgency WRITE setUrgency NOTIFY urgencyChanged)
+    /**
+     * @copydoc setAutoDelete
+     * @since 5.88
+     */
+    Q_PROPERTY(bool autoDelete READ isAutoDelete WRITE setAutoDelete NOTIFY autoDeleteChanged)
+    /**
+     * @since 5.90
+     */
+    Q_PROPERTY(QString xdgActivationToken READ xdgActivationToken NOTIFY xdgActivationTokenChanged)
+    /**
+     * @copydoc setHint
+     * @since 5.101
+     */
+    Q_PROPERTY(QVariantMap hints READ hints WRITE setHints NOTIFY hintsChanged)
 
 public:
-    /**
-     * Sometimes the user may want different notifications for the same event,
-     * depending the source of the event.  Example, you want to be notified for mails
-     * that arrive in your folder "personal inbox" but not for those in "spam" folder
-     *
-     * A notification context is a pair of two strings.
-     * The first string is a key from what the context is.  example "group" or
-     * "filter" (not translated).
-     * The second is the id of the context. In our example, the group id or the
-     * filter id in the applications.
-     * These strings are the ones present in the config file, and are in theory not
-     * shown in the user interface.
-     *
-     * The order of contexts in the list is is important, the most important context
-     * should be placed first. They are processed in that order when the notification occurs.
-     *
-     * @see event
-     */
-    typedef QPair<QString, QString> Context;
-    typedef QList< Context > ContextList;
-
     /**
      * @see NotificationFlags
      */
     enum NotificationFlag {
-        /**
-         * When the notification is activated, raise the notification's widget.
-         *
-         * This will change the desktop, raise the window, and switch to the tab.
-         * @todo  doesn't work yet
-         */
-        RaiseWidgetOnActivation = 0x01,
-
         /**
          * The notification will be automatically closed after a timeout. (this is the default)
          */
@@ -75,18 +167,6 @@ public:
          */
         Persistent = 0x02,
 
-        /**
-         * The notification will be automatically closed if the widget() becomes
-         * activated.
-         *
-         * If the widget is already activated when the notification occurs, the
-         * notification will be closed after a small timeout.
-         *
-         * This only works if the widget is the toplevel widget
-         * @todo make it work with tabulated widget
-         */
-        CloseWhenWidgetActivated = 0x04,
-
         /**
          * The audio plugin will loop the sound until the notification is closed
          */
@@ -99,11 +179,21 @@ public:
          */
         SkipGrouping = 0x10,
 
+        /**
+         * The notification will be automatically closed if the window() becomes
+         * activated.
+         *
+         * You need to set a window using setWindow().
+         *
+         * @since 6.0
+         */
+        CloseWhenWindowActivated = 0x20,
+
         /**
          * @internal
          * The event is a standard kde event, and not an event of the application
          */
-        DefaultEvent = 0xF000
+        DefaultEvent = 0xF000,
 
     };
 
@@ -111,11 +201,17 @@ public:
      * Stores a combination of #NotificationFlag values.
      */
     Q_DECLARE_FLAGS(NotificationFlags, NotificationFlag)
+    Q_FLAG(NotificationFlags)
 
     /**
      * default events you can use in the event function
      */
-    enum StandardEvent { Notification, Warning, Error, Catastrophe };
+    enum StandardEvent {
+        Notification,
+        Warning,
+        Error,
+        Catastrophe,
+    };
 
     /**
      * The urgency of a notification.
@@ -128,29 +224,9 @@ public:
         LowUrgency = 10,
         NormalUrgency = 50,
         HighUrgency = 70,
-        CriticalUrgency = 90
+        CriticalUrgency = 90,
     };
-
-#if KNOTIFICATIONS_ENABLE_DEPRECATED_SINCE(5, 75)
-    /**
-     * Create a new notification.
-     *
-     * You have to use sendEvent to show the notification.
-     *
-     * The pointer is automatically deleted when the event is closed.
-     *
-     * Make sure you use one of the NotificationFlags CloseOnTimeOut or
-     * CloseWhenWidgetActivated, if not,
-     * you have to close the notification yourself.
-     *
-     * @param eventId is the name of the event
-     * @param widget is a widget where the notification reports to
-     * @param flags is a bitmask of NotificationFlag
-     * @deprecated Since 5.75, use other constructor and call setWidget() explicitly
-     */
-    KNOTIFICATIONS_DEPRECATED_VERSION(5, 75, "Use other constructor and call setWidget() explicitly")
-    explicit KNotification(const QString &eventId, QWidget *widget, const NotificationFlags &flags = CloseOnTimeout);
-#endif
+    Q_ENUM(Urgency)
 
     /**
      * Create a new notification.
@@ -159,65 +235,25 @@ public:
      *
      * The pointer is automatically deleted when the event is closed.
      *
-     * The NotificationFlags is set to CloseOnTimeout.
-     *
-     * @param eventId is the name of the event
-     * @since 5.75
-     */
-    inline explicit KNotification(const QString &eventId)
-        : KNotification(eventId, CloseOnTimeout, nullptr)
-    {
-    }
-    // TODO KF6: remove thic constructor, in favour of other non-deprecated constructor
-    // It is a helper constructor for software binary compatibility of code which called the
-    // deprecated constructor so far due to it's second parameter "widget" having had
-    // a default nullptr value.
-
-    /**
-     * Create a new notification.
-     *
-     * You have to use sendEvent to show the notification.
-     *
-     * The pointer is automatically deleted when the event is closed.
-     *
-     * Make sure you use one of the NotificationFlags CloseOnTimeOut or
-     * CloseWhenWidgetActivated, if not,
-     * you have to close the notification yourself.
-     *
      * @since 4.4
      *
      * @param eventId is the name of the event
      * @param flags is a bitmask of NotificationFlag
      * @param parent parent object
      */
-    explicit KNotification(const QString &eventId, const NotificationFlags &flags, QObject *parent = nullptr);
-    // TODO KF6: pass flags by value instead of reference, set CloseOnTimeout as default value
+    explicit KNotification(const QString &eventId, NotificationFlags flags = CloseOnTimeout, QObject *parent = nullptr);
 
-    ~KNotification();
-
-    /**
-     * @brief the widget associated to the notification
-     *
-     * If the widget is destroyed, the notification will be automatically canceled.
-     * If the widget is activated, the notification will be automatically closed if the NotificationFlags specify that
-     *
-     * When the notification is activated, the widget might be raised.
-     * Depending on the configuration, the taskbar entry of the window containing the widget may blink.
-     */
-    QWidget *widget() const;
-
-    /**
-     * Set the widget associated to the notification.
-     * The notification is reparented to the new widget.
-     * \see widget()
-     * @param widget the new widget
-     */
-    void setWidget(QWidget *widget);
+    ~KNotification() override;
 
     /**
      * @return the name of the event
      */
     QString eventId() const;
+    /**
+     * Set the event id, if not already passed to the constructor.
+     * @since 5.88
+     */
+    void setEventId(const QString &eventId);
 
     /**
      * @return the notification title
@@ -282,15 +318,15 @@ public:
     void setPixmap(const QPixmap &pix);
 
     /**
-     * @return the default action, or an empty string if not set
-     * @since 5.31
+     * @return the default action, or nullptr if none is set
+     * @since 6.0
      */
-    QString defaultAction() const;
+    KNotificationAction *defaultAction() const;
 
     /**
-     * Set a default action that will be triggered when the notification is
+     * Add a default action that will be triggered when the notification is
      * activated (typically, by clicking on the notification popup). The default
-     * action should raise a window belonging to the application that sent it.
+     * action typically raises a window belonging to the application that sent it.
      *
      * The string will be used as a label for the action, so ideally it should
      * be wrapped in i18n() or tr() calls.
@@ -299,56 +335,63 @@ public:
      * In Plasma and Gnome desktops, the actions are performed by clicking on
      * the notification popup, and the label is not presented to the user.
      *
+     * Calling this overrides the current default action
      *
-     * @param action Label of the default action. The label might or might not
-     * be displayed to the user by the notification server, depending on the
-     * implementation. Passing an empty string disables the default action.
-     * @since 5.31
+     * @since 6.0
      */
-    void setDefaultAction(const QString &defaultAction);
+    [[nodiscard]] KNotificationAction *addDefaultAction(const QString &label);
 
     /**
-     * @return the list of actions
+     * Add an action to the notification.
+     *
+     * The visual representation of actions depends
+     * on the notification server.
+     *
+     * @param label the user-visible label of the action
+     *
+     * @see KNotificationAction
+     *
+     * @since 6.0
      */
-    //KF6: Rename to "additionalActions"?
-    QStringList actions() const;
+    [[nodiscard]] KNotificationAction *addAction(const QString &label);
 
     /**
-     * Set the list of actions shown in the popup. The strings passed
-     * in that QStringList will be used as labels for those actions,
-     * so ideally they should be wrapped in i18n() or tr() calls.
-     * In Plasma workspace, these will be shown as buttons inside
-     * the notification popup.
+     * Removes all actions previously added by addAction()
+     * from the notification.
      *
-     * The visual representation of actions however depends
-     * on the notification server
+     * @see addAction
      *
-     * @param actions List of strings used as action labels
+     * @since 6.0
      */
-    //KF6: Rename to "setAdditionalActions"?
-    void setActions(const QStringList &actions);
+    void clearActions();
 
     /**
-     * @return the list of contexts, see KNotification::Context
+     * @return the inline reply action.
+     * @since 5.81
      */
-    ContextList contexts() const;
+    KNotificationReplyAction *replyAction() const;
+
     /**
-     * set the list of contexts, see KNotification::Context
+     * @brief Add an inline reply action to the notification.
      *
-     * The list of contexts must be set before calling sendEvent;
+     * On supported platforms this lets the user type a reply to a notification,
+     * such as replying to a chat message, from the notification popup, for example:
+     *
+     * @code{.cpp}
+     * KNotification *notification = new KNotification(QStringLiteral("notification"));
+     * ...
+     * auto replyAction = std::make_unique<KNotificationReplyAction>(i18nc("@action:button", "Reply"));
+     * replyAction->setPlaceholderText(i18nc("@info:placeholder", "Reply to Dave..."));
+     * QObject::connect(replyAction.get(), &KNotificationReplyAction::replied, [](const QString &text) {
+     *     qDebug() << "you replied with" << text;
+     * });
+     * notification->setReplyAction(std::move(replyAction));
+     * @endcode
+     *
+     * @param replyAction the reply action to set
+     * @since 5.81
      */
-    void setContexts(const ContextList &contexts);
-    /**
-     * append a context at the list of contexts, see KNotificaiton::Context
-     * @param context the context which is added
-     */
-    void addContext(const Context &context);
-    /**
-     * @overload
-     * @param context_key is the key of the context
-     * @param context_value is the value of the context
-     */
-    void addContext(const QString &context_key, const QString &context_value);
+    void setReplyAction(std::unique_ptr<KNotificationReplyAction> replyAction);
 
     /**
      * @return the notification flags.
@@ -361,6 +404,11 @@ public:
      */
     void setFlags(const NotificationFlags &flags);
 
+    /**
+     * Returns the component name used to determine the location of the configuration file.
+     * @since 5.88
+     */
+    QString componentName() const;
     /**
      * The componentData is used to determine the location of the config file.
      *
@@ -413,10 +461,20 @@ public:
     void setUrgency(Urgency urgency);
 
     /**
-     * @internal
-     * the id given by the notification manager
+     * Sets the window associated with this notification.
+     * This is relevant when using the CloseWhenWindowActivated flag.
+     *
+     * @since 6.0
      */
-    int id();
+    void setWindow(QWindow *window);
+
+    /**
+     * The window associated with this notification. nullptr by default.
+     * @return the window set by setWindow()
+     *
+     * @since 6.0
+     */
+    QWindow *window() const;
 
     /**
      * @internal
@@ -424,47 +482,25 @@ public:
      */
     QString appName() const;
 
+    /**
+     * Returns whether this notification object will be automatically deleted after closing.
+     * @since 5.88
+     */
+    bool isAutoDelete() const;
+    /**
+     * Sets whether this notification object will be automatically deleted after closing.
+     * This is on by default for C++, and off by default for QML.
+     * @since 5.88
+     */
+    void setAutoDelete(bool autoDelete);
+
+    /**
+     * Returns the activation token to use to activate a window.
+     * @since 5.90
+     */
+    QString xdgActivationToken() const;
+
 Q_SIGNALS:
-#if KNOTIFICATIONS_ENABLE_DEPRECATED_SINCE(5, 76)
-    /**
-     * Emitted only when the default activation has occurred
-     * @deprecated Since 5.67, use defaultActivated() instead
-     */
-    KNOTIFICATIONS_DEPRECATED_VERSION_BELATED(5, 76, 5, 67, "Use defaultActivated() instead")
-    void activated();
-#endif
-
-    /**
-     * Emitted when the default action has been activated.
-     * @since 5.67
-     */
-    void defaultActivated();
-
-    /**
-     * Emitted when an action has been activated.
-     *
-     * The parameter passed by the signal is the index of the action
-     * in the QStringList set by setActions() call.
-     *
-     * @param action will be 0 if the default action was activated, or the index of the action in the actions QStringList
-     */
-    void activated(unsigned int action);
-
-    /**
-     * Convenience signal that is emitted when the first action is activated.
-     */
-    void action1Activated();
-
-    /**
-     * \overload
-     */
-    void action2Activated();
-
-    /**
-     * \overload
-     */
-    void action3Activated();
-
     /**
      * Emitted when the notification is closed.
      *
@@ -478,13 +514,73 @@ Q_SIGNALS:
      */
     void ignored();
 
-public Q_SLOTS:
     /**
-     * @brief Activate the action specified action
-     * If the action is zero, then the default action is activated
+     * Emitted when @c eventId changed.
+     * @since 5.88
      */
-    void activate(unsigned int action = 0);
+    void eventIdChanged();
+    /**
+     * Emitted when @c title changed.
+     * @since 5.88
+     */
+    void titleChanged();
+    /**
+     * Emitted when @c text changed.
+     * @since 5.88
+     */
+    void textChanged();
+    /**
+     * Emitted when @c iconName changed.
+     * @since 5.88
+     */
+    void iconNameChanged();
+    /**
+     * Emitted when @c defaultAction changed.
+     * @since 5.88
+     */
+    void defaultActionChanged();
+    /**
+     * Emitted when @c actions changed.
+     * @since 5.88
+     */
+    void actionsChanged();
+    /**
+     * Emitted when @p flags changed.
+     * @since 5.88
+     */
+    void flagsChanged();
+    /**
+     * Emitted when @p componentName changed.
+     * @since 5.88
+     */
+    void componentNameChanged();
+    /**
+     * Emitted when @p urls changed.
+     * @since 5.88
+     */
+    void urlsChanged();
+    /**
+     * Emitted when @p urgency changed.
+     * @since 5.88
+     */
+    void urgencyChanged();
+    /**
+     * Emitted when @p autoDelete changed.
+     * @since 5.88
+     */
+    void autoDeleteChanged();
+    /**
+     * Emitted when @p xdgActivationToken changes.
+     * @since 5.90
+     */
+    void xdgActivationTokenChanged();
+    /**
+     * Emitted when @p hints changes.
+     * @since 5.101
+     */
+    void hintsChanged();
 
+public Q_SLOTS:
     /**
      * Close the notification without activating it.
      *
@@ -492,34 +588,6 @@ public Q_SLOTS:
      */
     void close();
 
-#if KNOTIFICATIONS_ENABLE_DEPRECATED_SINCE(5, 67)
-    /**
-     * @brief Raise the widget.
-     * This will change the desktop, activate the window, and the tab if needed.
-     * @deprecated since 5.67, use QWindow raise + requestActivate instead.
-     */
-    KNOTIFICATIONS_DEPRECATED_VERSION(5, 67, "Use QWindow raise + requestActivate instead")
-    void raiseWidget();
-#endif
-
-    /**
-     * The notification will automatically be closed if all presentations are finished.
-     * if you want to show your own presentation in your application, you should use this
-     * function, so it will not be automatically closed when there is nothing to show.
-     *
-     * Don't forgot to deref, or the notification may be never closed if there is no timeout.
-     *
-     * @see deref
-     */
-    void ref();
-    /**
-     * Remove a reference made with ref(). If the ref counter hits zero,
-     * the notification will be closed and deleted.
-     *
-     * @see ref
-     */
-    void deref();
-
     /**
      * Send the notification to the server.
      *
@@ -528,19 +596,14 @@ public Q_SLOTS:
      */
     void sendEvent();
 
-    /**
-     * @internal
-     * update the texts, the icon, and the actions of one existing notification
-     */
-    void update();
-
     /**
      * @since 5.57
-     * Adds a custom hint to the notification. Those are key-value pairs that can be interpreted by the respective notification backend to trigger additional, non-standard features.
+     * Adds a custom hint to the notification. Those are key-value pairs that can be interpreted by the respective notification backend to trigger additional,
+     * non-standard features.
      * @param hint the hint's key
      * @param value the hint's value
      */
-    void setHint(const QString &hint, const QVariant &value);
+    Q_INVOKABLE void setHint(const QString &hint, const QVariant &value);
 
     /**
      * @since 5.57
@@ -548,18 +611,74 @@ public Q_SLOTS:
      */
     QVariantMap hints() const;
 
-private:
-    struct Private;
-    Private *const d;
-
-protected:
     /**
-     * reimplemented for internal reasons
+     * @since 5.101
+     * Set custom hints on the notification.
+     * @sa setHint
      */
-    bool eventFilter(QObject *watched, QEvent *event) override;
+    void setHints(const QVariantMap &hints);
+
+private:
+    friend class KNotificationManager;
+    friend class NotificationWrapper;
+    friend class NotifyByPopup;
+    friend class NotifyByPortal;
+    friend class NotifyByPortalPrivate;
+    friend class NotifyByExecute;
+    friend class NotifyBySnore;
+    friend class NotifyByAndroid;
+    friend class NotifyByMacOSNotificationCenter;
+    struct Private;
+
+    KNOTIFICATIONS_NO_EXPORT void slotWindowActiveChanged();
+
+    /**
+     * @brief Activate the action specified action
+     * If the action is zero, then the default action is activated
+     */
+    KNOTIFICATIONS_NO_EXPORT void activate(const QString &action);
+
+    /**
+     * @internal
+     * the id given by the notification manager
+     */
+    KNOTIFICATIONS_NO_EXPORT int id();
+
+    /**
+     * @internal
+     * update the texts, the icon, and the actions of one existing notification
+     */
+    KNOTIFICATIONS_NO_EXPORT void update();
+
+    /**
+     * The notification will automatically be closed if all presentations are finished.
+     * if you want to show your own presentation in your application, you should use this
+     * function, so it will not be automatically closed when there is nothing to show.
+     *
+     * Don't forgot to deref, or the notification may be never closed if there is no timeout.
+     *
+     * @see deref
+     */
+    KNOTIFICATIONS_NO_EXPORT void ref();
+
+    /**
+     * Remove a reference made with ref(). If the ref counter hits zero,
+     * the notification will be closed and deleted.
+     *
+     * @see ref
+     */
+    KNOTIFICATIONS_NO_EXPORT void deref();
+
+    // Like setActions, but doesn't take ownership
+    void setActionsQml(QList<KNotificationAction *> actions);
+    void setDefaultActionQml(KNotificationAction *action);
+    QList<KNotificationAction *> actions() const;
+
     static QString standardEventToEventId(StandardEvent event);
     static QString standardEventToIconName(StandardEvent event);
 
+    std::unique_ptr<Private> const d;
+
 public:
     /**
      * @brief emit an event
@@ -572,22 +691,18 @@ public:
      * @return a KNotification .  You may use that pointer to connect some signals or slot.
      * the pointer is automatically deleted when the event is closed.
      *
-     * Make sure you use one of the CloseOnTimeOut or CloseWhenWidgetActivated, if not,
-     * you have to close yourself the notification.
-     *
-     * @note the text is shown in a QLabel, you should escape HTML, if needed.
-     *
      * @param eventId is the name of the event
      * @param title is title of the notification to show in the popup.
      * @param text is the text of the notification to show in the popup.
      * @param pixmap is a picture which may be shown in the popup.
-     * @param widget is a widget where the notification reports to
      * @param flags is a bitmask of NotificationFlag
      * @param componentName used to determine the location of the config file.  by default, appname is used
      * @since 4.4
      */
-    static KNotification *event(const QString &eventId, const QString &title, const QString &text,
-                                const QPixmap &pixmap = QPixmap(), QWidget *widget = nullptr,
+    static KNotification *event(const QString &eventId,
+                                const QString &title,
+                                const QString &text,
+                                const QPixmap &pixmap = QPixmap(),
                                 const NotificationFlags &flags = CloseOnTimeout,
                                 const QString &componentName = QString());
 
@@ -601,12 +716,12 @@ public:
      * @param eventId is the name of the event
      * @param text is the text of the notification to show in the popup.
      * @param pixmap is a picture which may be shown in the popup.
-     * @param widget is a widget where the notification reports to
      * @param flags is a bitmask of NotificationFlag
      * @param componentName used to determine the location of the config file.  by default, plasma_workspace is used
      */
-    static KNotification *event(const QString &eventId, const QString &text = QString(),
-                                const QPixmap &pixmap = QPixmap(), QWidget *widget = nullptr,
+    static KNotification *event(const QString &eventId,
+                                const QString &text = QString(),
+                                const QPixmap &pixmap = QPixmap(),
                                 const NotificationFlags &flags = CloseOnTimeout,
                                 const QString &componentName = QString());
 
@@ -620,12 +735,10 @@ public:
      * @param eventId is the name of the event
      * @param text is the text of the notification to show in the popup
      * @param pixmap is a picture which may be shown in the popup
-     * @param widget is a widget where the notification reports to
      * @param flags is a bitmask of NotificationFlag
      */
-    static KNotification *event(StandardEvent eventId, const QString &text = QString(),
-                                const QPixmap &pixmap = QPixmap(), QWidget *widget = nullptr,
-                                const NotificationFlags &flags = CloseOnTimeout);
+    static KNotification *
+    event(StandardEvent eventId, const QString &text = QString(), const QPixmap &pixmap = QPixmap(), const NotificationFlags &flags = CloseOnTimeout);
 
     /**
      * @brief emit a standard event
@@ -638,13 +751,11 @@ public:
      * @param title is title of the notification to show in the popup.
      * @param text is the text of the notification to show in the popup
      * @param pixmap is a picture which may be shown in the popup
-     * @param widget is a widget where the notification reports to
      * @param flags is a bitmask of NotificationFlag
      * @since 4.4
      */
-    static KNotification *event(StandardEvent eventId, const QString &title, const QString &text,
-                                const QPixmap &pixmap, QWidget *widget = nullptr,
-                                const NotificationFlags &flags = CloseOnTimeout);
+    static KNotification *
+    event(StandardEvent eventId, const QString &title, const QString &text, const QPixmap &pixmap, const NotificationFlags &flags = CloseOnTimeout);
 
     /**
      * @brief emit a standard event with the possibility of setting an icon by icon name
@@ -657,13 +768,14 @@ public:
      * @param title is title of the notification to show in the popup.
      * @param text is the text of the notification to show in the popup
      * @param iconName a Freedesktop compatible icon name to be shown in the popup
-     * @param widget is a widget where the notification reports to
      * @param flags is a bitmask of NotificationFlag
      * @param componentName used to determine the location of the config file.  by default, plasma_workspace is used
      * @since 5.4
      */
-    static KNotification *event(const QString &eventId, const QString &title, const QString &text,
-                                const QString &iconName, QWidget *widget = nullptr,
+    static KNotification *event(const QString &eventId,
+                                const QString &title,
+                                const QString &text,
+                                const QString &iconName,
                                 const NotificationFlags &flags = CloseOnTimeout,
                                 const QString &componentName = QString());
 
@@ -678,13 +790,11 @@ public:
      * @param title is title of the notification to show in the popup.
      * @param text is the text of the notification to show in the popup
      * @param iconName a Freedesktop compatible icon name to be shown in the popup
-     * @param widget is a widget where the notification reports to
      * @param flags is a bitmask of NotificationFlag
      * @since 5.9
      */
-    static KNotification *event(StandardEvent eventId, const QString &title, const QString &text,
-                                const QString &iconName, QWidget *widget = nullptr,
-                                const NotificationFlags &flags = CloseOnTimeout);
+    static KNotification *
+    event(StandardEvent eventId, const QString &title, const QString &text, const QString &iconName, const NotificationFlags &flags = CloseOnTimeout);
 
     /**
      * @brief emit a standard event
@@ -696,23 +806,19 @@ public:
      * @param eventId the type of the standard (not app-defined) event
      * @param title is title of the notification to show in the popup.
      * @param text is the text of the notification to show in the popup
-     * @param widget is a widget where the notification reports to
      * @param flags is a bitmask of NotificationFlag
      * @since 5.9
      */
-    static KNotification *event(StandardEvent eventId, const QString &title, const QString &text,
-                                QWidget *widget = nullptr, const NotificationFlags &flags = CloseOnTimeout);
-
+    static KNotification *event(StandardEvent eventId, const QString &title, const QString &text, const NotificationFlags &flags = CloseOnTimeout);
 
     /**
      * This is a simple substitution for QApplication::beep()
      *
      * @param reason a short text explaining what has happened (may be empty)
-     * @param widget the widget the notification refers to
      */
-    static void beep(const QString &reason = QString(), QWidget *widget = nullptr);
+    static void beep(const QString &reason = QString());
 
-    //prevent warning
+    // prevent warning
     using QObject::event;
 };
 
